/*
  Fluorescent Led Several

  This gives a nice effect with lot of individual fluorescent tubes in a hall
  as we want to have several tubes, we use a array of objects

  Fluorescent lamps come often in tube form.
  the might have some flicker in the beginning.
  after the flickering the reach around 80% and will need some time for full brightness

  you must connect the LED to a PWM enabled PIN 

  by noiasca
  2022-01-17
*/

#include <Noiasca_led.h>               // download library from http://werner.rothschopf.net/microcontroller/202202_tools_led_en.htm
#include <utility/Noiasca_discrete.h>  // use the discrete pins on the Arduino/Microcontroller
#include <Noiasca_button.h>            // some tools to read buttons

FluorescentPin fluorescentLed[]{3, 5, 6, 9, 10, 11};       // UNO PWM pins 3, 5, 6, 9, 10, 11. On non PWM pins, you will only see a "blink" at start
Button button[] {A0, A1, A2};          // button connects pin with GND, "active LOW"

void setup() {
  Serial.begin(115200);
  randomSeed(analogRead(A6));
  for (auto &obj : fluorescentLed) {
    obj.begin();                       // you have to call the .begin() method for the LED
    obj.on();                          // you can switch the LED on
  }
  
  for (auto &obj : button)
  {
    obj.begin();   // you have to call the .begin() method for each button
  }
}

void loop() {
  // read the buttons
  if (button[0].wasPressed()) {
    Serial.println(F("button 0 was pressed, switch ON LED"));
    for (auto &obj : fluorescentLed) obj.on();   // switch on LED
  }
  if (button[1].wasPressed()) {
    Serial.println(F("button 1 was pressed, switch OFF LED"));
    for (auto &obj : fluorescentLed) obj.off(); // switch off LED
  }
  if (button[2].wasPressed()) {
    Serial.println(F("button 2 was pressed, toogle LED (change from ON to OFF or vice versa)"));
    for (auto &obj : fluorescentLed) obj.toggle(); // if button was pressed, toggle (change state) of blinkLed
  }

  for (auto &obj : fluorescentLed) obj.update(); // you have to call update() for the LED
}
