/*
  Turn Signal

  simulate the turn signal of a car
  beside OFF there are 3 state: LEFT, RIGHT or HAZARD
  where HAZARD means, both sides are blinking (and an optional hazard warning light in the dashboard)
  
  by noiasca
  2022-01-17   2962/219
  2022-01-16   2906/232
  
*/

#include <Noiasca_led.h>               // download library from http://werner.rothschopf.net/microcontroller/202202_tools_led_en.htm
#include <utility/Noiasca_discrete.h>  // use the discrete pins on the Arduino/Microcontroller
#include <Noiasca_button.h>            // some tools to read buttons

TurnsignalPin turnsignalPin{2, 3, 4};  // GPIO left, GPIO right, GPIO hazard warning light (optional)
// TurnsignalPin turnsignalPin {2, 3}; // GPIO left, GPIO right
Button button[] {A0, A1, A2, A3};      // button connects pin with GND, "active LOW"

void setup() {
  Serial.begin(115200);
  turnsignalPin.begin();               // you have to call the .begin() method for the LED pair
  turnsignalPin.left();                // you can switch the LED on

//  turnsignal.setInterval(500, 500);  // you can modify the delay time for the pulsing. Default value is 50.
 
  for (auto &obj : button) {
    obj.begin();                       // you have to call the .begin() method for each button
  }
}

void loop() {
  // read the buttons
  if (button[0].wasPressed()) {
    Serial.println(F("button 0 was pressed, switch OFF"));
    turnsignalPin.off();               // switch on LED
  }
  if (button[1].wasPressed()) {
    Serial.println(F("button 1 was pressed, turn LEFT"));
    turnsignalPin.left();              // switch off LED
  }
  if (button[2].wasPressed()) {
    Serial.println(F("button 2 was pressed, turn RIGHT"));
    turnsignalPin.right();             // if button was pressed, toggle (change state) of LED
  }
  if (button[3].wasPressed()) {
    Serial.println(F("button 3 was pressed, hazard light"));
    turnsignalPin.hazard();            // if button was pressed, toggle (change state) of LED
  }

  turnsignalPin.update();   // you have to call update() for the LED
}
