/*
  Traffic light combined

  a traffic light for cars and a traffic light for pedestrains.
  The timer of car traffic light is used as client (master) to control,
  the pedestrain traffic light acts as server (slave) and reacts on the client

  by noiasca
  2021-12-04
*/

#include <Noiasca_led.h>                         // download library from http://werner.rothschopf.net/microcontroller/202202_tools_led_en.htm
#include <utility/Noiasca_discrete.h>            // use the discrete pins on the Arduino/Microcontroller
#include <Noiasca_button.h>                      // some tools to read buttons

TrafficlightPin trafficlight {2, 3, 5};          // GPIO red, GPIO yellow, GPIO green
TrafficlightPin trafficwalk {6, 255, 7};         // GPIO red, (yellow not used), GPIO green
Button button[] {A0, A1, A2, A3};                // button connects pin with GND, "active LOW"

// a callback function receives state changes from the car traffic light and keeps the pedestrain trafficlight in sync
void cbOnStateChange(byte newState)
{
  Serial.print(F("state changed to ")); Serial.println(newState);
  // everytime we get a new state from trafficlight we adopt the pedestrain traffic light
  switch (newState) {
    case TrafficlightPin::State::RED : // if the car traffic light reports red 
      trafficwalk.green();             // we can switch the pedestrain to green
      break;
    default :
      trafficwalk.red();               // in all other cases we just switch to red
  }
}

void setup() {
  Serial.begin(115200);
  trafficlight.begin();                // you have to call the .begin() method for the LED pair
  trafficwalk.begin();

  trafficlight.yellowBlink();          // you can switch each lamp on individually
  trafficwalk.red();

  for (auto &obj : button) {
    obj.begin();   // you have to call the .begin() method for each button
  }

  // optional member functions
  trafficwalk.setMode(TrafficlightPin::Mode::MANUAL);  // we change the pedestrain light to manual
  trafficlight.setOnStateChange(cbOnStateChange);   // register the callback function to the event of trafficlight changes
}

void loop() {
  // read the buttons
  if (button[0].wasPressed()) {
    Serial.println(F("button 0 was pressed"));
    trafficlight.off();  // switch on LED
  }
  if (button[1].wasPressed()) {
    Serial.println(F("button 1 was pressed"));
    trafficlight.red(); // switch off LED
  }
  if (button[2].wasPressed()) {
    Serial.println(F("button 2 was pressed"));
    trafficlight.green(); // if button was pressed, toggle (change state) of LED
  }
  if (button[3].wasPressed()) {
    Serial.println(F("button 3 was pressed"));
    trafficlight.yellow(); // if button was pressed, toggle (change state) of LED
  }
  trafficlight.update();   // you have to call update() for the lights
  trafficwalk.update();
}
