/*
  Traffic light crossing

  Two traffic lights for cars (red, yellow, green) and two traffic light for pedestrains (red green)
  All traffic lights are controlled by an external traffic manager.

  by noiasca
  2022-01-17 nok - needs rework after last update
*/

#include <Noiasca_led.h>                         // download library from http://werner.rothschopf.net/microcontroller/202202_tools_led_en.htm
#include <utility/Noiasca_discrete.h>            // use the discrete pins on the Arduino/Microcontroller
#include <Noiasca_button.h>                      // some tools to read buttons

TrafficlightPin carsVertical {2, 3, 4};          // GPIO red, GPIO yellow, GPIO green
TrafficlightPin pedestrainVertical {5, 255, 6};  // GPIO red, (yellow not used), GPIO green
TrafficlightPin carsHorizontal {7, 8, 9};        // GPIO red, GPIO yellow, GPIO green
TrafficlightPin pedestrainHorizontal {10, 255, 11}; // GPIO red, (yellow not used), GPIO green
Button button[] {A0, A1};                        // button connects pin with GND, "active LOW"

class Trafficmanager {
    unsigned long previousMillis = -1;  // last state timestamp
    byte currentSequence = 0;
    struct Sequence {
      uint16_t interval;
      byte carsV;
      byte carsH;
      byte pedestrainV;
      byte pedestrainH;
    };

    static const size_t noOfSequence = 4;
    Sequence sequence[noOfSequence] {
      {5000, TrafficlightPin::State::RED,       TrafficlightPin::State::GREEN,     TrafficlightPin::State::GREEN, TrafficlightPin::State::RED},
      {3000, TrafficlightPin::State::REDYELLOW, TrafficlightPin::State::YELLOW,    TrafficlightPin::State::RED,   TrafficlightPin::State::RED},
      {2500, TrafficlightPin::State::GREEN,     TrafficlightPin::State::RED,       TrafficlightPin::State::RED,   TrafficlightPin::State::GREEN},
      {3000, TrafficlightPin::State::YELLOW,    TrafficlightPin::State::REDYELLOW, TrafficlightPin::State::RED,   TrafficlightPin::State::RED}
    };

  public:
    Trafficmanager() {}

    void setSequence(byte newState) {
      carsVertical.setState(sequence[newState].carsV);
      carsHorizontal.setState(sequence[newState].carsH);
      pedestrainVertical.setState(sequence[newState].pedestrainV);
      pedestrainHorizontal.setState(sequence[newState].pedestrainH);
      currentSequence = newState;
    }

    void update() {
      uint32_t currentMillis = millis();
      if (currentMillis - previousMillis >= sequence[currentSequence].interval) {
        previousMillis = currentMillis;
        setSequence(currentSequence);
        currentSequence++;
        if (currentSequence >= noOfSequence) currentSequence = 0;
      }
      if (currentSequence == 2 && button[0].wasPressed()) {
        Serial.println(F("button 0 was pressed"));
        setSequence(3);
        previousMillis = currentMillis;
      }
      if (currentSequence == 0 && button[1].wasPressed()) {
        Serial.println(F("button 1 was pressed"));
        setSequence(1);
        previousMillis = currentMillis;
      }
    }
} trafficmanager;

void setup() {
  Serial.begin(115200);
  for (auto &obj : button) obj.begin();   // you have to call the .begin() method for each button
  carsVertical.begin();                   // you have to call the .begin() method for the traffic lights
  carsHorizontal.begin();
  pedestrainVertical.begin();
  pedestrainHorizontal.begin();
  // you can switch each lamp on individually
  carsVertical.yellowBlink();  
  carsHorizontal.yellowBlink();
  pedestrainVertical.red();
  pedestrainHorizontal.red();
  // all traffic lights in this example are controlled externally
  carsVertical.setMode(TrafficlightPin::Mode::MANUAL);
  carsHorizontal.setMode(TrafficlightPin::Mode::MANUAL);
  pedestrainVertical.setMode(TrafficlightPin::Mode::MANUAL);
  pedestrainHorizontal.setMode(TrafficlightPin::Mode::MANUAL);
}

void loop() {
  trafficmanager.update();
  carsVertical.update();
  carsHorizontal.update();
  pedestrainVertical.update();
  pedestrainHorizontal.update();
}
