/*
   Blink FastLED
   blinks 3 pixel on a strip using FastLED library

   by noiasca

   2022-04-16 ok
*/

// needed to control the FastLED strip:
#include <FastLED.h>                   // include the FastLED library (install via library manager)
constexpr uint16_t pixelCount = 4;     // how many pixels on your strip
constexpr byte pixelPin = 8;           // data pin of your strip
CRGB leds[pixelCount];                 // This is an array of leds. One item for each led in your strip.

#include <Noiasca_led.h>               // download from: https://werner.rothschopf.net/microcontroller/202202_tools_led_en.htm
#include <utility/Noiasca_fastled.h>   // needed to control FastLED 

// create your objects:
BlinkFastLED blinkFastLEDA(FastLED, 1, leds);      // a blink LED on a FastLED strip using a specific pixel
BlinkFastLED blinkFastLEDB(FastLED, 2, leds);      // a blink LED on a FastLED strip using a specific pixel
BlinkFastLED blinkFastLEDC(FastLED, 3, leds);      // a blink LED on a FastLED strip using a specific pixel

// create Objects for your pixels - optional without the wrapper:
//Fastled_IF fastled_HW(FastLED, 1, leds);  
//Blink<Fastled_IF> blinkFastLEDA(fastled_HW);

void setup() {
  Serial.begin(115200);
  Serial.println(F("LED & Pixel Test"));
  // init your strip according the FastLED example:
  FastLED.addLeds<WS2811, pixelPin, RGB>(leds, pixelCount);
  leds[0] = CRGB::White; FastLED.show(); // you can still access LEDs on the strip in the common "FastLED" way
  FastLED.setBrightness(50); // Set BRIGHTNESS to about 1/5 (max = 255)
  blinkFastLEDA.setOnInterval(250);      // set the on time of a pixel
  //blinkFastLEDB.on();                  // you can switch on or off a pixel
  //blinkFastLEDB.setOffInterval(250);   // set the off time of a pixel
  blinkFastLEDB.setOnColor(0xFFFF00);    // set the on color of a pixel
  blinkFastLEDC.setOnColor(0x00FF00);
  blinkFastLEDC.setOffColor(0x000088);   // the "off" color can be different to black
}

void loop() {
  blinkFastLEDA.update();                // call .update() for your pixel object
  blinkFastLEDB.update();
  blinkFastLEDC.update();
}
