/*
  Blink several LEDs independently
  This example uses a PCA9685 / PWM Servo Driver for up to 16 LEDs

  by noiasca
  2022-12-17    5492/318
*/

#include <Noiasca_led.h>               // download library from http://werner.rothschopf.net/microcontroller/202202_tools_led_en.htm
#include <utility/Noiasca_PCA9685.h>   // needed for PWM Servo Driver
#include <Wire.h>                      // I2C library for the PCA9685
#include <Adafruit_PWMServoDriver.h>   // install with Library Manager
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver(); // create a pwm instance

BlinkPCA9685 blinkLed[] {
  {pwm, 0},  // declare several blinking LED and assign pins on your PWM object
  {pwm, 1},
  {pwm, 2},
  {pwm, 3},
  {pwm, 4},
  {pwm, 5}
};

void setup() {
  Wire.begin();                                  // start I2C
  pwm.begin();                                   // start PCA9685
  //pwm.setOscillatorFrequency(27000000);        // optional PCA9685 set frequency
  //pwm.setPWMFreq(60);                          // optional PCA PWM frequency between 40 and 1600
                                                 
  for (auto &obj : blinkLed)                     
  {                                              
    obj.begin();                                 // you have to call the .begin() method for each LED object
    obj.on();                                    // you can switch the blinking on in the setup already
  }

  // you can set different blink intervals for your LEDs
  blinkLed[0].setOnInterval(700);
  blinkLed[1].setOnInterval(200);
  blinkLed[2].setOffInterval(500);
  blinkLed[3].setOffInterval(400);
}

void loop() {
  for (auto &obj : blinkLed) obj.update();       // you have to call update() for each LED object
  // put other non blocking code here, to run repeatedly
}
