/*
  Heartbeat LED
  dimms up and down.
  Can be used for a "heartbeat" LED

  This example uses a PCA9685 / PWM Servo Driver for up to 16 LEDs
  
  by noiasca
  2022-12-17 NOK
*/

#include <Noiasca_led.h>               // download library from http://werner.rothschopf.net/microcontroller/202202_tools_led_en.htm
#include <utility/Noiasca_PCA9685.h>   // needed for PWM Servo Driver
#include <Noiasca_button.h>            // some tools to read buttons
#include <Wire.h>                      // I2C library for the PCA9685
#include <Adafruit_PWMServoDriver.h>   // install with Library Manager
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver(); // create a pwm instance

HeartbeatPCA9685 heartbeatLed {pwm, 1};// one pin on your pwm object
Button button[] {A0, A1, A2};          // button connects pin with GND, "active LOW"

void setup() {
  Serial.begin(115200);                          // start Serial interface
  Wire.begin();                                  // start I2C
  pwm.begin();                                   // start PCA9685
  //pwm.setOscillatorFrequency(27000000);        // optional PCA9685 set frequency
  //pwm.setPWMFreq(60);                          // optional PCA PWM frequency between 40 and 1600
 
  heartbeatLed.begin();                // you have to call the .begin() method for the LED
  heartbeatLed.on();                   // you can switch the LED on
  heartbeatLed.setInterval(10);        // you can modify the delay time for the pulsing. Default value is 50.
  heartbeatLed.setMinBrightness(50);   // you can limit the mininum brightness. Default value is 0
  heartbeatLed.setMaxBrightness(240);  // you can limit the maximum brightness. Default value is 255. Highest value on Arduino is 255. 
  
  for (auto &obj : button)
  {
    obj.begin();   // you have to call the .begin() method for each button
  }
}

void loop() {
  // read the buttons
  if (button[0].wasPressed())
  {
    Serial.println(F("button 0 was pressed, switch ON LED"));
    heartbeatLed.on();       // switch on LED
  }
  if (button[1].wasPressed()) {
    Serial.println(F("button 1 was pressed, switch OFF LED"));
    heartbeatLed.off();      // switch off LED
  }
  if (button[2].wasPressed()) {
    Serial.println(F("button 2 was pressed, toogle LED (change from ON to OFF or vice versa)"));
    heartbeatLed.toggle();   // if button was pressed, toggle (change state) of LED
  }
  heartbeatLed.update();     // you have to call update() for the LED
}
