/*
  Smooth LED with Timer

  on of several LEDs dimm up, is on for a specific time and then dimms donw.0

  This example uses a PCA9685 / PWM Servo Driver for up to 16 LEDs

  by noiasca
  2022-12-18 OK
*/

#include <Noiasca_led.h>               // download library from https://werner.rothschopf.net/microcontroller/202202_tools_led_en.htm
#include <utility/Noiasca_PCA9685.h>   // needed for PWM Servo Driver
#include <Noiasca_timer.h>             // contained in the Noiasca Tool Kit 

#include <Wire.h>                      // I2C library for the PCA9685
#include <Adafruit_PWMServoDriver.h>   // install with Library Manager
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver(); // create a pwm instance

SmoothPCA9685 smoothLed[]              // one output on your pwm object
{
  {pwm, 0},        // first parameter your PWMServoDriver instance, second parameter the pin on that instance
  {pwm, 1},
  {pwm, 2},
  {pwm, 3},
  {pwm, 4},
  {pwm, 5}
};

constexpr size_t noOfLeds = sizeof(smoothLed) / sizeof(smoothLed[0]);// calculate the number of leds in the array

// this is a callback function
// here your define, what the timer should do
// when the time interval is over and the event gets triggered
void cbOnInterval() {
  Serial.println(F("timer fired"));
  static uint8_t actual = 0;           // the actual active LED. static as the variable should survive the runtime of this function
  smoothLed[actual].off();
  actual++;
  if (actual >= noOfLeds) actual = 0;  // handle rollover
  smoothLed[actual].on();
}

// here you create your timer object
//                       callback     interval in milliseconds
LittleTimer littleTimer {cbOnInterval, 5000};

void setup() {
  Serial.begin(115200);                          // start Serial interface
  Wire.begin();                                  // start I2C
  pwm.begin();                                   // start PCA9685
  //pwm.setOscillatorFrequency(27000000);        // optional PCA9685 set frequency
  //pwm.setPWMFreq(60);                          // optional PCA PWM frequency between 40 and 1600

  for (auto &obj : smoothLed)
  {
    obj.begin();                                  // you have to call the .begin() method for the LED
    obj.off();                                    // in this example we have to switch off the LED
    obj.setOnInterval(30);                        // optional: you can modify the delay time for the smoothing
    obj.setOffInterval(5);                        // optional: you can modify the delay time for the smoothing
    //obj.setMaxBrightness(240);                  // optional: you can limit the maximum brightness. Highest value on Arduino is 255. Default 255.
  }                                               
  smoothLed[0].on();                              // switch first LED on
  littleTimer.start();                            // and start the timer
}

void loop() {
  uint32_t currentMillis = millis();              // this will save some millis() calls per loop
  for (auto &obj : smoothLed)                     
  {                                               
    obj.update(currentMillis);                    // you have to call update() for the LED - we reuse the currentMillis
  }                                               
  littleTimer.update(currentMillis);              // if the defined time is over, the timer will trigger the callback function
}
