/*
  Timer simple

  this example shows how to use a simple timer

  it will let a LED blink for 3 seconds and pause for 3 seconds
  
  by noiasca
  2021-08-27 2398/231 callback version
*/

#include <Noiasca_led.h>               // download library from http://werner.rothschopf.net/microcontroller/202202_tools_led_en.htm
#include <utility/Noiasca_discrete.h>  // use the discrete pins on the Arduino/Microcontroller
#include <Noiasca_button.h>            // some tools to read buttons
#include <Noiasca_timer.h>             // simple timer for time management

BlinkPin blinkLed {13};

// this is a callback function
// here your define, what the timer should do
// when the time intervall is over and the event gets triggered
void cbOnInterval() {
  blinkLed.toggle();
}

// here you create your timer object
// you need two parameters:
// the first is the callback to your function
// the second paraemter is the interval
// the timer will call this functioon in that interval
//                       callback     interval in milliseconds
LittleTimer littleTimer {cbOnInterval, 3000};

void setup() {
  Serial.begin(115200);
  Serial.println(F("\nStart"));
  blinkLed.begin();                    // start a LED object
  blinkLed.on();                       // switch the LED object on 
  littleTimer.start();                 // and start the timer
}

void loop() {
  // you must call the update() method for each object:
  blinkLed.update();
  littleTimer.update();                // if the defined time is over, the timer will trigger the callback function
}
