/*
  Timer with limit

  A timer can have a limit, to stop the timer after n iterations
  and you can assign callback function for the start and for the end

  this example is based on this requeset: https://forum.arduino.cc/t/ausgang-schalten-nach-signal/899726/8

  by noiasca
*/

#include <Noiasca_led.h>               // download library from http://werner.rothschopf.net/microcontroller/202202_tools_led_en.htm
#include <utility/Noiasca_discrete.h>  // use the discrete pins on the Arduino/Microcontroller
#include <Noiasca_timer.h>             // here we have some tools

const byte inputPin = A0;              // sensor to be checked 
const byte openPin = 9;                // relay pin to open the door
const byte closePin = 8;               // relay pin to close the door
const byte ledPin = 13;                // a LED indicating that the gate moves

BlinkPin led {ledPin};

void cbOpenOn() {
  Serial.println(F("open on"));
  digitalWrite(openPin, HIGH);
  digitalWrite(closePin, LOW);
  led.on();
}

void cbOpenOff() {
  Serial.println(F("open off"));
  digitalWrite(openPin, LOW);
  led.off();
}

void cbCloseOn() {
  Serial.println(F("close on"));
  digitalWrite(openPin, LOW);
  digitalWrite(closePin, HIGH);
  led.on();
}

void cbCloseOff() {
  Serial.println(F("close off"));
  digitalWrite(closePin, LOW);
  led.off();
}

//                    callback  interval  limit
LittleTimer openTimer {cbOpenOff,  3000,   1};
LittleTimer closeTimer{cbCloseOff, 3000,   1};

void cbReadButton() {
  static byte previousState = 0;
  byte currentState = digitalRead(inputPin);
  if (currentState != previousState) {
    Serial.println(F("state change"));
    if (currentState == HIGH)
      openTimer.start();
    else
      closeTimer.start();
    previousState = currentState;
  }
}
LittleTimer inputTimer{cbReadButton, 20}; // for debounce

void setup() {
  Serial.begin(115200);
  Serial.println(F("\nStart"));
  pinMode(inputPin, INPUT_PULLUP);
  pinMode(openPin, OUTPUT);
  pinMode(closePin, OUTPUT);
  led.begin();
  openTimer.attachOnStart(cbOpenOn);
  closeTimer.attachOnStart(cbCloseOn);
  inputTimer.start();
}

void loop() {
  // you must call the update() method for each object:
  led.update();
  openTimer.update();
  closeTimer.update();
  inputTimer.update();
}
