/*
    This sketch compiles lot of different LED objects
    It is only used for testing
    it might not compile in every version

    2023-08-04 ok 17668/1190 
    2023-07-10 ok 17664/1190 
    2023-02-18 ok 17432/1190 after adding HT16K33
    2022-12-19 ok 16678/1143 after adding PCF8574 and PCA9685 (and Wire.h)
    2022-12-19 ok 11748/807
    2022-04-11 ok 11800/807
    2022-02-15 ok 10900/762
    2022-01-17 ok 10594/734
*/
#include <Noiasca_led.h>

// discrete Arduino pins
#include <utility/Noiasca_discrete.h>  // use the discrete pins on the Arduino/Microcontroller

BlinkPin blinkPin(13);                // a blink LED on a discrete Arduino pin
Bounce5Pin bounce5Pin(3, 4, 5, 6, 7);
OnOffPin onoffPin(9);                 // a simple on/off LED (no effect)

/*
    use the Adafruit library for Neopixel
*/
#include <utility/Noiasca_neopixel.h>  // needed for Neopixel
#include <Adafruit_NeoPixel.h>
constexpr uint16_t pixelCount = 32;
constexpr byte pixelPin = 8;
Adafruit_NeoPixel strip(pixelCount, pixelPin, NEO_GRB + NEO_KHZ800);

BlinkPixel blinkPixelA(strip, 0);     // a blink LED on a Neopixel strip
RhythmPixel rhythmPixelB(strip, 1);   // another blink LED on a Neopixel strip
PulsePixel pulsePixelC(strip, 2);     // a monoflop pixel on for a short time
HeartbeatPixel heartbeatPixelD(strip, 3);
FlickerPixel flickerPixelE(strip, 4);
FluorescentPixel fluorescentPixelF(strip, 5);
SmoothPixel smoothPixelG(strip, 6);
Bounce5Pixel bounce5Pixel(strip, 7);   // needs pixel 7, 8, 9, 10, 11
AlternatingPixel alternatingPixel(strip, 12); // needs pixel 12, 13
TurnsignalPixel turnsignalPixel(strip, 16, 17, 18);
TrafficlightPixel trafficlightPixel(strip, 19, 20, 21);

// use FastLED for Neopixel - only include so it gets compiled - LEDs missing
#include <utility/Noiasca_fastled.h>

#include <Wire.h>                      // I2C library for the HT16K33, PCA9685, PCF8574 etc

#include <utility/Noiasca_HT16K33.h>   // needed to control HT16K33 - part of the LED tool kit
HT16K33expander myHT16K33(0x76);       // create an expander instance at a specific I2C address (usually between 0x70 - 0x77)
BlinkHT16K33 blinkHT16K33(myHT16K33, 0 + 0);     // a blink LED on a port expander  (first cathode, first Anode)

//  use the Adafruit library for PCA9685
#include <utility/Noiasca_PCA9685.h>   // needed for PWM Servo Driver PCA9685
#include <Adafruit_PWMServoDriver.h>   // install with Library Manager
Adafruit_PWMServoDriver myPCA9685 = Adafruit_PWMServoDriver(); // create a pwm instance
BlinkPCA9685 blinkPCA9685[] {
  {myPCA9685, 0},  // declare several blinking LED and assign pins on your PWM object
  {myPCA9685, 1},
  {myPCA9685, 2}
};

// use PCF8574 expander object
#include <utility/Noiasca_PCF8574.h>             // needed to control PCF8574 
PCF8574expander myPCF8574(0x3F);                 // create a PCF8574 instance at a specific I2C address
BlinkPCF8574 blinkPCF8574(myPCF8574, 3);         // could be the backlight pin of a LCD I2C backpack

/*
    example how to use the "complex" interface without wrapper
*/
NeoPixel lastPixel (strip, pixelCount - 1 - 3);
//Blink<NeoPixel> last(lastPixel);

NeoPixelGroup<3> lastGroup (strip, 29, 30, 31);
Trafficlight<NeoPixelGroup<3>> last(lastGroup);

//Blink<NeoPixel> blinkC(lastPixel); OK

//style    <hardware> name(object)

//Bounce5<NeoPixel> last(lastPixel);

// test
//DiscreteGroup5 group5(3, 4, 5, 6, 7);
//Bounce5<DiscreteGroup5> last(group5);

//DiscreteGroup<5> group5(3, 4, 5, 6, 7);
//Bounce5<DiscreteGroup<5>> last(group5);

//DiscreteGroup<3> group3(3, 4, 5);
//Bounce5<DiscreteGroup<3>> last(group3);

//DiscreteGroup<5> group5(3, 4, 5, 6, 7);

//DiscretePin myPin(9);
//OnOff<DiscretePin> onoffPin(myPin);

void setup() {
  Serial.begin(115200);
  Serial.println(F("LED & Pixel Test"));

  blinkPin.begin();                        // objects for Arduino pins need the begin() method
  onoffPin.begin();

  strip.begin();
  strip.setBrightness(50);  // Set BRIGHTNESS to about 1/5 (max = 255)
  onoffPin.on();
  blinkPixelA.setOnInterval(250);
  rhythmPixelB.setOnColor(0x000080);
  pulsePixelC.setOnInterval(2000);
  pulsePixelC.on();
  heartbeatPixelD.setOnColor(0xFF4000);
  flickerPixelE.setOnColor(0xFFAA00);
  fluorescentPixelF.on();
  smoothPixelG.setOnColor(0x00FF00);
  bounce5Pin.begin();
  bounce5Pixel.setOnColor(0x800000);
  alternatingPixel.begin();
  alternatingPixel.setOnColor(0x000080);
  turnsignalPixel.setOnColor(0, 0x808000);   // color for left turn signal
  turnsignalPixel.setOnColor(1, 0x808000);   // color for right turn signal
  turnsignalPixel.setOnColor(2, 0x800000);   // color for hazard light in dashboard
  turnsignalPixel.hazard();                  // .left(), right(), or .hazard()
  trafficlightPixel.setOnColor(0, 0xFF0000); // color for red traffic signal
  trafficlightPixel.setOnColor(1, 0xFF8000); // color for yellow traffic signal
  trafficlightPixel.setOnColor(2, 0x008000); // color for green traffic signal

  Wire.begin();                                  // start I2C - for several I2C expanders

  myHT16K33.begin();               // start the HT16K33 object
  blinkHT16K33.begin();            // start the HT16K33 LED
  
  myPCA9685.begin();                               // start PCA9685
  for (auto &obj : blinkPCA9685) {
    obj.begin();                                 // you have to call the .begin() method for each LED object
    obj.on();                                    // you can switch the blinking on in the setup already
  }

  last.begin();
  //last.setOnColor(0x800000);
  //last.on();
  //last.left();
}

void loop() {
  // put your main code here, to run repeatedly:
  blinkPin.update();
  bounce5Pin.update();
  onoffPin.update();
  blinkPixelA.update();
  rhythmPixelB.update();
  pulsePixelC.update();
  heartbeatPixelD.update();
  flickerPixelE.update();
  fluorescentPixelF.update();
  smoothPixelG.update();
  bounce5Pixel.update();
  alternatingPixel.update();
  turnsignalPixel.update();
  trafficlightPixel.update();
  for (auto &obj : blinkPCA9685) obj.update();

  blinkPCF8574.update();
  blinkHT16K33.update();

  last.update();
}
